@@ -13,7 +13,6 @@
 #    under the License.
 
 import copy
-import re
 
 import netaddr
 from octavia_lib.api.drivers import data_models as o_datamodels
@@ -38,12 +37,16 @@ LOG = logging.getLogger(__name__)
 class OvnProviderDriver(driver_base.ProviderDriver):
 
     def __init__(self):
-        super(OvnProviderDriver, self).__init__()
+        super().__init__()
         self._ovn_helper = ovn_helper.OvnProviderHelper()
 
     def __del__(self):
         self._ovn_helper.shutdown()
 
+    def _is_health_check_supported(self):
+        return self._ovn_helper.ovn_nbdb_api.is_col_present(
+            'Load_Balancer', 'health_check')
+
     def _check_for_supported_protocols(self, protocol):
         if protocol not in ovn_const.OVN_NATIVE_LB_PROTOCOLS:
             msg = _('OVN provider does not support %s protocol') % protocol
@@ -81,6 +84,18 @@ class OvnProviderDriver(driver_base.ProviderDriver):
                    'info': request_info}
         self._ovn_helper.add_request(request)
 
+        if not isinstance(loadbalancer.listeners, o_datamodels.UnsetType):
+            for listener in loadbalancer.listeners:
+                self.listener_create(listener)
+
+        if not isinstance(loadbalancer.pools, o_datamodels.UnsetType):
+            for pool in loadbalancer.pools:
+                self.pool_create(pool)
+                for member in pool.members:
+                    if not member.subnet_id:
+                        member.subnet_id = loadbalancer.vip_network_id
+                    self.member_create(member)
+
     def loadbalancer_delete(self, loadbalancer, cascade=False):
         request_info = {'id': loadbalancer.loadbalancer_id,
                         'cascade': cascade}
@@ -89,10 +104,10 @@ class OvnProviderDriver(driver_base.ProviderDriver):
         self._ovn_helper.add_request(request)
 
     def loadbalancer_failover(self, loadbalancer_id):
-        request_info = {'id': loadbalancer_id}
-        request = {'type': ovn_const.REQ_TYPE_LB_FAILOVER,
-                   'info': request_info}
-        self._ovn_helper.add_request(request)
+        msg = _('OVN provider does not support loadbalancer failover')
+        raise driver_exceptions.UnsupportedOptionError(
+            user_fault_string=msg,
+            operator_fault_string=msg)
 
     def loadbalancer_update(self, old_loadbalancer, new_loadbalancer):
         request_info = {'id': new_loadbalancer.loadbalancer_id}
@@ -149,7 +164,6 @@ class OvnProviderDriver(driver_base.ProviderDriver):
     def listener_create(self, listener):
         self._check_for_supported_protocols(listener.protocol)
         self._check_for_allowed_cidrs(listener.allowed_cidrs)
-
         admin_state_up = listener.admin_state_up
         if isinstance(admin_state_up, o_datamodels.UnsetType):
             admin_state_up = True
@@ -200,28 +214,42 @@ class OvnProviderDriver(driver_base.ProviderDriver):
             return True
         return False
 
+    def _check_member_monitor_options(self, member):
+        if self._check_monitor_options(member):
+            msg = _('OVN Load Balancer does not support different member '
+                    'monitor address or port.')
+            raise driver_exceptions.UnsupportedOptionError(
+                user_fault_string=msg,
+                operator_fault_string=msg)
+
     def _ip_version_differs(self, member):
         _, ovn_lb = self._ovn_helper._find_ovn_lb_by_pool_id(member.pool_id)
+        if not ovn_lb:
+            return False
         lb_vip = ovn_lb.external_ids[ovn_const.LB_EXT_IDS_VIP_KEY]
         return netaddr.IPNetwork(lb_vip).version != (
             netaddr.IPNetwork(member.address).version)
 
     def member_create(self, member):
-        if self._check_monitor_options(member):
-            msg = _('OVN provider does not support monitor options')
-            raise driver_exceptions.UnsupportedOptionError(
-                user_fault_string=msg,
-                operator_fault_string=msg)
+        # Validate monitoring options if present
+        self._check_member_monitor_options(member)
         if self._ip_version_differs(member):
             raise ovn_exc.IPVersionsMixingNotSupportedError()
         admin_state_up = member.admin_state_up
-        if (isinstance(member.subnet_id, o_datamodels.UnsetType) or
-                not member.subnet_id):
-            msg = _('Subnet is required for Member creation '
-                    'with OVN Provider Driver')
-            raise driver_exceptions.UnsupportedOptionError(
-                user_fault_string=msg,
-                operator_fault_string=msg)
+        subnet_id = member.subnet_id
+        if (isinstance(subnet_id, o_datamodels.UnsetType) or not subnet_id):
+            subnet_id, subnet_cidr = self._ovn_helper._get_subnet_from_pool(
+                member.pool_id)
+            if not (subnet_id and
+                    self._ovn_helper._check_ip_in_subnet(member.address,
+                                                         subnet_cidr)):
+                msg = _('Subnet is required, or Loadbalancer associated with '
+                        'Pool must have a subnet, for Member creation '
+                        'with OVN Provider Driver if it is not the same as '
+                        'LB VIP subnet')
+                raise driver_exceptions.UnsupportedOptionError(
+                    user_fault_string=msg,
+                    operator_fault_string=msg)
 
         if isinstance(admin_state_up, o_datamodels.UnsetType):
             admin_state_up = True
@@ -229,7 +257,7 @@ class OvnProviderDriver(driver_base.ProviderDriver):
                         'address': member.address,
                         'protocol_port': member.protocol_port,
                         'pool_id': member.pool_id,
-                        'subnet_id': member.subnet_id,
+                        'subnet_id': subnet_id,
                         'admin_state_up': admin_state_up}
         request = {'type': ovn_const.REQ_TYPE_MEMBER_CREATE,
                    'info': request_info}
@@ -241,18 +269,37 @@ class OvnProviderDriver(driver_base.ProviderDriver):
         request_info = {'id': member.member_id,
                         'address': member.address,
                         'pool_id': member.pool_id,
-                        'subnet_id': member.subnet_id,
+                        'subnet_id': subnet_id,
                         'action': ovn_const.REQ_INFO_MEMBER_ADDED}
         request = {'type': ovn_const.REQ_TYPE_HANDLE_MEMBER_DVR,
                    'info': request_info}
         self._ovn_helper.add_request(request)
 
     def member_delete(self, member):
+        # NOTE(froyo): OVN provider allow to create member without param
+        # subnet_id, in that case the driver search it according to the
+        # pool_id, but it is not propagated to Octavia. In this case, if
+        # the member is deleted, Octavia send the object without subnet_id.
+        subnet_id = member.subnet_id
+        if (isinstance(subnet_id, o_datamodels.UnsetType) or not subnet_id):
+            subnet_id, subnet_cidr = self._ovn_helper._get_subnet_from_pool(
+                member.pool_id)
+            if not (subnet_id and
+                    self._ovn_helper._check_ip_in_subnet(member.address,
+                                                         subnet_cidr)):
+                msg = _('Subnet is required, or Loadbalancer associated with '
+                        'Pool must have a subnet, for Member deletion if it is'
+                        'with OVN Provider Driver if it is not the same as '
+                        'LB VIP subnet')
+                raise driver_exceptions.UnsupportedOptionError(
+                    user_fault_string=msg,
+                    operator_fault_string=msg)
+
         request_info = {'id': member.member_id,
                         'address': member.address,
                         'protocol_port': member.protocol_port,
                         'pool_id': member.pool_id,
-                        'subnet_id': member.subnet_id}
+                        'subnet_id': subnet_id}
         request = {'type': ovn_const.REQ_TYPE_MEMBER_DELETE,
                    'info': request_info}
         self._ovn_helper.add_request(request)
@@ -262,25 +309,42 @@ class OvnProviderDriver(driver_base.ProviderDriver):
         request_info = {'id': member.member_id,
                         'address': member.address,
                         'pool_id': member.pool_id,
-                        'subnet_id': member.subnet_id,
+                        'subnet_id': subnet_id,
                         'action': ovn_const.REQ_INFO_MEMBER_DELETED}
         request = {'type': ovn_const.REQ_TYPE_HANDLE_MEMBER_DVR,
                    'info': request_info}
         self._ovn_helper.add_request(request)
 
     def member_update(self, old_member, new_member):
-        if self._check_monitor_options(new_member):
-            msg = _('OVN provider does not support monitor options')
-            raise driver_exceptions.UnsupportedOptionError(
-                user_fault_string=msg,
-                operator_fault_string=msg)
+        # NOTE(froyo): OVN provider allow to create member without param
+        # subnet_id, in that case the driver search it according to the
+        # pool_id, but it is not propagated to Octavia. In this case, if
+        # the member is updated, Octavia send the object without subnet_id.
+        subnet_id = old_member.subnet_id
+        if (isinstance(subnet_id, o_datamodels.UnsetType) or not subnet_id):
+            subnet_id, subnet_cidr = self._ovn_helper._get_subnet_from_pool(
+                old_member.pool_id)
+            if not (subnet_id and
+                    self._ovn_helper._check_ip_in_subnet(new_member.address,
+                                                         subnet_cidr)):
+                msg = _('Subnet is required, or Loadbalancer associated with '
+                        'Pool must have a subnet, for Member update '
+                        'with OVN Provider Driver if it is not the same as '
+                        'LB VIP subnet')
+                raise driver_exceptions.UnsupportedOptionError(
+                    user_fault_string=msg,
+                    operator_fault_string=msg)
+
+        # Validate monitoring options if present
+        self._check_member_monitor_options(new_member)
         if new_member.address and self._ip_version_differs(new_member):
             raise ovn_exc.IPVersionsMixingNotSupportedError()
         request_info = {'id': new_member.member_id,
                         'address': old_member.address,
                         'protocol_port': old_member.protocol_port,
                         'pool_id': old_member.pool_id,
-                        'subnet_id': old_member.subnet_id}
+                        'subnet_id': subnet_id,
+                        'old_admin_state_up': old_member.admin_state_up}
         if not isinstance(new_member.admin_state_up, o_datamodels.UnsetType):
             request_info['admin_state_up'] = new_member.admin_state_up
         request = {'type': ovn_const.REQ_TYPE_MEMBER_UPDATE,
@@ -292,42 +356,49 @@ class OvnProviderDriver(driver_base.ProviderDriver):
         skipped_members = []
         pool_key, ovn_lb = self._ovn_helper._find_ovn_lb_by_pool_id(pool_id)
         external_ids = copy.deepcopy(ovn_lb.external_ids)
-        existing_members = external_ids[pool_key].split(',')
+        pool = external_ids[pool_key]
+        existing_members = pool.split(',') if pool else []
         members_to_delete = copy.copy(existing_members)
+        pool_subnet_id = None
         for member in members:
             if (self._check_monitor_options(member) or
                     member.address and self._ip_version_differs(member)):
                 skipped_members.append(member.member_id)
                 continue
-            # NOTE(mjozefcz): We need to have subnet_id information.
-            if (isinstance(member.subnet_id, o_datamodels.UnsetType) or
-                    not member.subnet_id):
-                msg = _('Subnet is required for Member creation '
-                        'with OVN Provider Driver')
-                raise driver_exceptions.UnsupportedOptionError(
-                    user_fault_string=msg,
-                    operator_fault_string=msg)
+            # NOTE(froyo): if subnet_id not provided, lets try to get it
+            # from the member pool_id
+            subnet_id = member.subnet_id
+            if (isinstance(subnet_id, o_datamodels.UnsetType) or
+                    not subnet_id):
+                if not pool_subnet_id:
+                    pool_subnet_id, pool_subnet_cidr = (
+                        self._ovn_helper._get_subnet_from_pool(pool_id))
+                if pool_subnet_id:
+                    if (self._ovn_helper._check_ip_in_subnet(
+                            member.address, pool_subnet_cidr)):
+                        member.subnet_id = pool_subnet_id
+                # NOTE(mjozefcz): We need to have subnet_id information.
+                if not member.subnet_id:
+                    msg = _('Subnet is required, or Loadbalancer associated '
+                            'with Pool must have a subnet, for Member '
+                            'batch update with OVN Provider Driver if it is '
+                            'not the same as LB VIP subnet')
+                    raise driver_exceptions.UnsupportedOptionError(
+                        user_fault_string=msg,
+                        operator_fault_string=msg)
+
             admin_state_up = member.admin_state_up
             if isinstance(admin_state_up, o_datamodels.UnsetType):
                 admin_state_up = True
 
-            member_info = self._ovn_helper._get_member_key(member)
-            # TODO(mjozefcz): Remove this workaround in W release.
-            member_info_old = self._ovn_helper._get_member_key(
-                member, old_convention=True)
-            member_found = [x for x in existing_members
-                            if re.match(member_info_old, x)]
-            if not member_found:
+            member_info = self._ovn_helper._get_member_info(member)
+            if member_info not in existing_members:
                 req_type = ovn_const.REQ_TYPE_MEMBER_CREATE
             else:
                 # If member exists in pool, then Update
                 req_type = ovn_const.REQ_TYPE_MEMBER_UPDATE
                 # Remove all updating members so only deleted ones are left
-                # TODO(mjozefcz): Remove this workaround in W release.
-                try:
-                    members_to_delete.remove(member_info_old)
-                except ValueError:
-                    members_to_delete.remove(member_info)
+                members_to_delete.remove(member_info)
 
             request_info = {'id': member.member_id,
                             'address': member.address,
@@ -341,12 +412,14 @@ class OvnProviderDriver(driver_base.ProviderDriver):
 
         for member in members_to_delete:
             member_info = member.split('_')
+            member_ip, member_port, subnet_id = (
+                self._ovn_helper._extract_member_info(member)[0])
             request_info = {'id': member_info[1],
-                            'address': member_info[2].split(':')[0],
-                            'protocol_port': member_info[2].split(':')[1],
+                            'address': member_ip,
+                            'protocol_port': member_port,
                             'pool_id': pool_id}
             if len(member_info) == 4:
-                request_info['subnet_id'] = member_info[3]
+                request_info['subnet_id'] = subnet_id
             request = {'type': ovn_const.REQ_TYPE_MEMBER_DELETE,
                        'info': request_info}
             request_list.append(request)
@@ -375,3 +448,66 @@ class OvnProviderDriver(driver_base.ProviderDriver):
             raise driver_exceptions.DriverError(
                 **kwargs)
         return vip_dict
+
+    def _validate_hm_support(self, hm, action='create'):
+        if not self._is_health_check_supported():
+            msg = _('OVN Load Balancer supports Health Check provider '
+                    'from version 2.12. Upgrade OVN in order to use it.')
+            raise driver_exceptions.UnsupportedOptionError(
+                user_fault_string=msg,
+                operator_fault_string=msg)
+        # type is only required for create
+        if action == 'create':
+            if isinstance(hm.type, o_datamodels.UnsetType):
+                msg = _('OVN provider health monitor type not specified.')
+                # seems this should be other than "unsupported"?
+                raise driver_exceptions.UnsupportedOptionError(
+                    user_fault_string=msg,
+                    operator_fault_string=msg)
+            if hm.type not in ovn_const.SUPPORTED_HEALTH_MONITOR_TYPES:
+                msg = (_('OVN provider does not support %s '
+                         'health monitor type. Supported types: %s') %
+                        (hm.type,
+                         ', '.join(ovn_const.SUPPORTED_HEALTH_MONITOR_TYPES)))
+                raise driver_exceptions.UnsupportedOptionError(
+                    user_fault_string=msg,
+                    operator_fault_string=msg)
+
+    def health_monitor_create(self, healthmonitor):
+        self._validate_hm_support(healthmonitor)
+        admin_state_up = healthmonitor.admin_state_up
+        if isinstance(admin_state_up, o_datamodels.UnsetType):
+            admin_state_up = True
+        request_info = {'id': healthmonitor.healthmonitor_id,
+                        'pool_id': healthmonitor.pool_id,
+                        'type': healthmonitor.type,
+                        'interval': healthmonitor.delay,
+                        'timeout': healthmonitor.timeout,
+                        'failure_count': healthmonitor.max_retries_down,
+                        'success_count': healthmonitor.max_retries,
+                        'admin_state_up': admin_state_up}
+        request = {'type': ovn_const.REQ_TYPE_HM_CREATE,
+                   'info': request_info}
+        self._ovn_helper.add_request(request)
+
+    def health_monitor_update(self, old_healthmonitor, new_healthmonitor):
+        self._validate_hm_support(new_healthmonitor, action='update')
+        admin_state_up = new_healthmonitor.admin_state_up
+        if isinstance(admin_state_up, o_datamodels.UnsetType):
+            admin_state_up = True
+        request_info = {'id': new_healthmonitor.healthmonitor_id,
+                        'pool_id': old_healthmonitor.pool_id,
+                        'interval': new_healthmonitor.delay,
+                        'timeout': new_healthmonitor.timeout,
+                        'failure_count': new_healthmonitor.max_retries_down,
+                        'success_count': new_healthmonitor.max_retries,
+                        'admin_state_up': admin_state_up}
+        request = {'type': ovn_const.REQ_TYPE_HM_UPDATE,
+                   'info': request_info}
+        self._ovn_helper.add_request(request)
+
+    def health_monitor_delete(self, healthmonitor):
+        request_info = {'id': healthmonitor.healthmonitor_id}
+        request = {'type': ovn_const.REQ_TYPE_HM_DELETE,
+                   'info': request_info}
+        self._ovn_helper.add_request(request)
